package test;

import java.io.File;

import net.sf.samtools.SAMFileReader;
import net.sf.samtools.SAMFileWriter;
import net.sf.samtools.SAMFileWriterFactory;
import net.sf.samtools.SAMRecord;
import net.sf.samtools.SAMFileReader.ValidationStringency;

public class EmitReads {

	public static void main(String[] args) {

		if (args.length != 2) {
			System.out.println("Remove reads with unmapped mates");
			System.out.println("Usage : input.bam output.bam");
			return;
		}
		
		File inputFile = new File(args[0]);
		SAMFileReader.setDefaultValidationStringency(ValidationStringency.LENIENT);
		final SAMFileReader inputSam = new SAMFileReader(inputFile);
		File outputFile = new File(args[1]);
		
	        // makeSAMorBAMWriter() writes a file in SAM text or BAM binary format depending
	        // on the file extension, which must be either .sam or .bam.

	        // Since the SAMRecords will be written in the same order as they appear in the input file,
	        // and the output file is specified as having the same sort order (as specified in
	        // SAMFileHeader.getSortOrder(), presorted == true.  This is much more efficient than
	        // presorted == false, if coordinate or queryname sorting is specified, because the SAMRecords
	        // can be written to the output file directly rather than being written to a temporary file
	        // and sorted after all records have been sent to outputSam.

	        final SAMFileWriter outputSam = new SAMFileWriterFactory().makeSAMOrBAMWriter(inputSam.getFileHeader(),
	                true, outputFile);

	        int readCount = 0;
	        int outReadCount = 0;
	        for (final SAMRecord samRecord : inputSam) {
	        	readCount++;
	            if ( !samRecord.getMateUnmappedFlag() && (!samRecord.getReadUnmappedFlag()) && (!samRecord.getReadFailsVendorQualityCheckFlag()) && (!samRecord.getDuplicateReadFlag())) {
	            	outputSam.addAlignment(samRecord);
	            	outReadCount++;
	            }
	        }

	        outputSam.close();
	        inputSam.close();
	        
	        System.out.println("Wrote " + outReadCount + " of " + readCount + " total reads");
	}
}
